void save(matrix_p m)
{
	u32_t r,c;
	for(r=0;r < m->rows;r++)
	{
		for(c=0;c<m->cols;c++)
			printf("%g\t",m->cell[r][c]);
		puts("");
	}
}

matrix_p load_image(char *name)
{
	u32_t w,h,i,j;
	matrix_p m;
	vec_t scale = 1.0 / 3 / 256;
	u8_t *ptr,*img = jpgLoad(name,&w,&h);
	if(!img)
		return NULL;
	m = matAlloc(h,w);
	ptr = img;
	for(i=0;i<m->rows;i++)
		for(j=0;j<m->cols;j++)
		{
			m->cell[i][j] = (ptr[0]+ptr[1]+ptr[2]) * scale;
			ptr += 4;
		}
	memFree(img);
	return m;
}

void save_image(char *name,matrix_p src)
{
	u32_t i,j;
	vec_t val;
	u8_t *ptr, *img = (u8_p)memAllocZ(src->rows * src->cols * 4);
	if(!img)
		return;
	ptr = img;
	for(i=0;i < src->rows;i++)
		for(j=0;j < src->cols;j++)
		{
			val = src->cell[i][j] * 255;
			if(val < 0)
				val = 0;
			else if(val > 255)
				val = 255;
			ptr[0] = ptr[1] = ptr[2] = (u8_t)val;
			ptr += 4;
		}
	bmpSave(name,src->cols,src->rows,img);
	memFree(img);
}

void
matRVMFTZeroCols(matrix_p rvmft,u32_t col_start,u32_t col_end)
{
	u32_t r, ft_len, row_bytes;
	V_ASSERT(rvmft);
	V_ASSERT(col_start <= col_end);
	/* Clamp the range to positive part of Fourier transform */
	ft_len = rvmft->cols >> 1;
	if(col_end >= ft_len)
		col_end = ft_len - 1;
	if(col_start == col_end)
	{
		for(r = 0;r < rvmft->rows;r++)
		{
			rvmft->cell[r][col_start << 1] = 0;
			rvmft->cell[r][(col_start << 1) + 1] = 0;
		}
	}
	else
	{
		row_bytes = (col_end - col_start + 1) * sizeof(vec_t) * 2;
		for(r = 0;r < rvmft->rows;r++)
			memZero(&rvmft->cell[r][col_start << 1],row_bytes);
	}
}

void
matScaleAdd(matrix_p dest,matrix_p src,vec_t scale,vec_t offset)
{
	u32_t r, c;
	V_ASSERT(dest->rows == src->rows);
	V_ASSERT(dest->cols == src->cols);
	for(r = 0;r < dest->rows;r++)
		for(c = 0;c < dest->cols;c++)
			dest->cell[r][c] = offset + src->cell[r][c] * scale;
}

vboolean
cspaceAdjustContrast(matrix_p dest,matrix_p src,vec_t min,vec_t max)
{
	vec_t rscale;
    rscale = max - min;
    if(!IsZero(rscale))
    {
    	rscale = 1 / rscale;
    	matScaleAdd(dest,src,rscale,- min * rscale);
    	return true;
    }
   	conlPuts(LOG_WARNING,"cspaceAdjustContrast(): zero input range, can't transform.\n");
    return false;
}

vboolean
cspaceMaximizeContrast(matrix_p m)
{
	u32_t r, c;
	vec_t min = 0, max = 0;
    matMin(m,&r,&c);
    min = m->cell[r][c];
    matMax(m,&r,&c);
    max = m->cell[r][c];
    return cspaceAdjustContrast(m,m,min,max);
}

matrix_p
matPSEofRVMFT(matrix_p rvmft)
{
	u32_t r, c;
	matrix_p row_pse = matAlloc(rvmft->rows >> 1,rvmft->cols >> 1);
	vec_p *cell = rvmft->cell;
	for(c = 0;c < row_pse->cols;c++)
		row_pse->cell[0][c] =
			2 * hypot(cell[0][c << 1],cell[0][(c << 1) + 1]);
	for(r = 1;r < row_pse->rows;r++)
	{
		u32_t neg_r = rvmft->rows - r;
		for(c = 0;c < row_pse->cols;c++)
			row_pse->cell[r][c] =
				hypot(cell[r][c << 1],cell[r][(c << 1) + 1]);
		for(c = 0;c < row_pse->cols;c++)
			row_pse->cell[r][c] +=
				hypot(cell[neg_r][c << 1],cell[neg_r][(c << 1) + 1]);
	}
	return row_pse;
}

void matAdd1(matrix_p m,vec_t val)
{
	u32_t r;
	vec_p ptr, endptr;
	for(r = 0;r < m->rows;r++)
	{
		ptr = m->cell[r];
		endptr = ptr + m->cols;
		for(;ptr < endptr;ptr++)
			*ptr += val;
	}
}

void matLn(matrix_p m)
{
	u32_t r;
	vec_p ptr, endptr;
	for(r = 0;r < m->rows;r++)
	{
		ptr = m->cell[r];
		endptr = ptr + m->cols;
		for(;ptr < endptr;ptr++)
			*ptr = log(*ptr);
	}
}

