#ifndef	V_DEBUG_H
#define	V_DEBUG_H

/*
 * Helper macros for output debug and diagnostic messages
 */

#ifdef	CHECKED_BUILD

/* declaration of static character string */
#define DEBUG_MESSAGE(identifier,message) \
	static char *identifier = message;

/* same as above, containing name of current function */
#define CALLER_NAME(name_of_routine) \
	DEBUG_MESSAGE(caller_name,#name_of_routine)

/* most widely used "incorrect parameters" message */
#define DEBUG_MESSAGE_INCORRECT() \
	DEBUG_MESSAGE(incorrect_parameters,"incorrect parameter(s)")

/*
 * Macros for printing error messages
 */
#define	ERROR_GENERIC(desc) \
	conlPrintf(LOG_ERROR,"%s(): ERROR: %s.\n",caller_name,desc);

#define	ERROR_GENERIC_2ARG(desc1,desc2) \
	conlPrintf(LOG_ERROR,"%s(): ERROR: %s%s.\n",caller_name,desc1,desc2);

#define	WARNING_GENERIC(desc) \
	conlPrintf(LOG_WARNING,"%s(): WARNING: %s.\n",caller_name,desc);

#define	ERROR_EOF() \
	conlPrintf(LOG_ERROR,"%s(): ERROR: unexpected EOF.\n",caller_name);

#define	ERROR_INVALID_STR(desc,got_str) \
	conlPrintf(LOG_ERROR,"%s(): ERROR: %s \"%s\".\n",caller_name,desc,strSafe(got_str));

#define	ERROR_INVALID_INT(desc,got_int) \
	conlPrintf(LOG_ERROR,"%s(): ERROR: %s \"%d\".\n",caller_name,desc,got_int);

#define	ERROR_INVALID_FLOAT(desc,got_float) \
	conlPrintf(LOG_ERROR,"%s(): ERROR: %s \"%g\".\n",caller_name,desc,got_float);

/*
 * Lexer - related error messages
 */
#define PARSE_ERROR(lexer,desc) \
	conlPrintf(LOG_ERROR, \
		"%s(): ERROR in \"%s\", line %u: %s (got \"%s\").\n", \
		caller_name,lexSourceName(lexer), \
		lexLastLine(lexer),desc,strSafe(lexLast(lexer)));

#define SYN_EXPECTED(lexer,desc) \
	conlPrintf(LOG_ERROR, \
		"%s(): ERROR in \"%s\", line %u: expected \"%s\", got \"%s\".\n", \
		caller_name,lexSourceName(lexer), \
		lexLastLine(lexer),desc,strSafe(lexLast(lexer)));

#define SYN_UNEXPECTED(lexer,token) \
	conlPrintf(LOG_ERROR, \
		"%s(): ERROR in \"%s\", line %u: unexpected token \"%s\".\n", \
		caller_name,lexSourceName(lexer),lexLastLine(lexer),(token));

#define SYN_UNEXPECTED_EOL(lexer) \
	conlPrintf(LOG_ERROR, \
		"%s(): ERROR in \"%s\", line %u: unexpected end of line.\n", \
		caller_name,lexSourceName(lexer),lexLastLine(lexer));

#define SYN_UNEXPECTED_EOF(lexer) \
	conlPrintf(LOG_ERROR, \
		"%s(): ERROR in \"%s\", line %u: unexpected end of file.\n", \
		caller_name,lexSourceName(lexer),lexLastLine(lexer));

#define SYN_UNDECLARED(lexer,desc) \
	conlPrintf(LOG_ERROR, \
		"%s(): ERROR in \"%s\", line %u: \"%s\" undeclared.\n", \
		caller_name,lexSourceName(lexer),lexLastLine(lexer),desc);

#else	/* !CHECKED_BUILD */

#define DEBUG_MESSAGE(identifier,message)
#define CALLER_NAME(name)
#define DEBUG_MESSAGE_INCORRECT()

#define	ERROR_GENERIC(desc)
#define	ERROR_GENERIC_2ARG(desc1,desc2)
#define	WARNING_GENERIC(desc)
#define	ERROR_EOF()
#define	ERROR_INVALID_STR(desc,got_str)
#define	ERROR_INVALID_INT(desc,got_int)
#define	ERROR_INVALID_FLOAT(desc,got_float)

#define PARSE_ERROR(lexer,desc)
#define	SYN_EXPECTED(lexer,desc)
#define SYN_UNEXPECTED_EOL(lexer)
#define SYN_UNEXPECTED_EOF(lexer)
#define SYN_UNDECLARED(lexer,desc)

#endif	/* !CHECKED_BUILD */

#endif /* !V_DEBUG_H */
